#include "i2c.h"

/*******************************************************************************
* Function Name  : I2C_Init
* Description    : I/OģIIC,PB4ΪSDA,PB7ΪSCL
* Input          : None			
* Return         : None
*******************************************************************************/
void I2C_Init(void)
{
	GPIOB_SetBits(GPIO_Pin_4|GPIO_Pin_7);
	GPIOB_ModeCfg(GPIO_Pin_4|GPIO_Pin_7, GPIO_ModeOut_PP_5mA);				//
}


/*******************************************************************************
* Function Name  : I2C_Start
* Description    : IIC
* Input          : None			
* Return         : None
*******************************************************************************/
void I2C_Start(void)
{
	SDA_OUT();
	I2C_SCL_HIGH();
	I2C_SDA_HIGH();
	DelayUs(4);
	I2C_SDA_LOW();
	DelayUs(4);
	I2C_SCL_LOW();
}


/*******************************************************************************
* Function Name  : I2C_Stop
* Description    : IICֹͣ
* Input          : None			
* Return         : None
*******************************************************************************/
void I2C_Stop(void)
{
	SDA_OUT();
	I2C_SCL_LOW();
	I2C_SDA_LOW();
	DelayUs(4);
	I2C_SCL_HIGH();
	I2C_SDA_HIGH();
	DelayUs(4);
}


/*******************************************************************************
* Function Name  : I2C_WaitAck
* Description    : ȴӻACK
* Input          : None			
* Return         : None
*******************************************************************************/
UINT8 I2C_WaitAck(void)
{
	UINT8 waittime = 0;
	SDA_IN();
	I2C_SDA_HIGH();
	DelayUs(1);
	I2C_SCL_HIGH();
	DelayUs(1);
	while(READ_SDA())
	{
		++waittime;
		if(waittime>250)
		{
			I2C_Stop();
			return 1;
		}
	}
	I2C_SCL_LOW();
	return 0;
}


/*******************************************************************************
* Function Name  : I2C_Ack
* Description    : ACK
* Input          : None			
* Return         : None
*******************************************************************************/
void I2C_Ack(void)
{
	I2C_SCL_LOW();
	SDA_OUT();
	I2C_SDA_LOW();
	DelayUs(2);
	I2C_SCL_HIGH();
	DelayUs(2);
	I2C_SCL_LOW();
}


/*******************************************************************************
* Function Name  : I2C_NAck
* Description    : NACK
* Input          : None			
* Return         : None
*******************************************************************************/
void I2C_NAck(void)
{
	I2C_SCL_LOW();
	SDA_OUT();
	I2C_SDA_HIGH();
	DelayUs(2);
	I2C_SCL_HIGH();
	DelayUs(2);
	I2C_SCL_LOW();
}

/*******************************************************************************
* Function Name  : I2C_SendByte
* Description    : IICһֽ
* Input          : txd: ׼͵ֽ			
* Return         : None
*******************************************************************************/
void I2C_SendByte(UINT8 txd)
{
	UINT8 i;
	SDA_OUT();
	I2C_SCL_LOW();
	DelayUs(2);
	
	for(i=0; i<8; ++i)
	{
		if(txd & 0x80)
			I2C_SDA_HIGH();
		else
			I2C_SDA_LOW();
			
		txd<<=1;
		DelayUs(2);
		I2C_SCL_HIGH();
		DelayUs(2);
		I2C_SCL_LOW();
		DelayUs(2);
	}
}

/*******************************************************************************
* Function Name  : I2C_ReceiveByte
* Description    : IICһֽ
* Input          : ack: 0		ǷظACK
											  1
* Return         : None
*******************************************************************************/
UINT8 I2C_ReceiveByte(UINT8 ack)
{
	UINT8 i, rxd=0;
	SDA_IN();
	I2C_SCL_LOW();
	DelayUs(2);
	
	for(i=0; i<8; ++i)
	{
		I2C_SCL_HIGH();
		rxd<<=1;
		if(READ_SDA())
			++rxd;
		DelayUs(1);
	}
	
	if(!ack)
		I2C_NAck();
	else
		I2C_Ack();
	
	return rxd;
}
