#include "oled.h"
#include "oledfont.h"

UINT8 OLED_GRAM[128][8];		//OLEDʾʱ

/*******************************************************************************
* Function Name  : OLED_WriteCmd
* Description    : OLEDд
* Input          : cmd: 			
* Return         : None
*******************************************************************************/
void OLED_WriteCmd(UINT8 cmd)
{
	I2C_Start();
	I2C_SendByte(0x78);
	I2C_WaitAck();
	I2C_SendByte(0x00);
	I2C_WaitAck();
	I2C_SendByte(cmd);
	I2C_WaitAck();
	I2C_Stop();
}


/*******************************************************************************
* Function Name  : OLED_WriteData
* Description    : OLEDд
* Input          : data: 			
* Return         : None
*******************************************************************************/
void OLED_WriteData(UINT8 data)
{
	I2C_Start();
	I2C_SendByte(0x78);
	I2C_WaitAck();
	I2C_SendByte(0x40);
	I2C_WaitAck();
	I2C_SendByte(data);
	I2C_WaitAck();
	I2C_Stop();
}


/*******************************************************************************
* Function Name  : OLED_RefreshGram
* Description    : ˢOLEDʾ
* Input          : None			
* Return         : None
*******************************************************************************/
void OLED_RefreshGram(void)
{
	signed char p;
	UINT8 x;
	for(p=7; p>=0; --p)
	{
		OLED_WriteCmd(0xB0+p);
		OLED_WriteCmd(0x00);
		OLED_WriteCmd(0x10);
		for(x=0; x<128; ++x)
		{
			OLED_WriteData(OLED_GRAM[x][p]);
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_Clear
* Description    : OLEDʾ
* Input          : None			
* Return         : None
*******************************************************************************/
void OLED_Clear(void)
{
	UINT8 x, p;
	for(p=0; p<8; ++p)
		for(x=0; x<128; ++x)
		{
			OLED_GRAM[x][p] = 0;
		}
		OLED_RefreshGram();
}


/*******************************************************************************
* Function Name  : OLED_DisplayOn
* Description    : OLEDʾ
* Input          : None			
* Return         : None
*******************************************************************************/
void OLED_DisplayOn(void)
{
	OLED_WriteCmd(0x8D);
	OLED_WriteCmd(0x14);
	OLED_WriteCmd(0xAF);
}


/*******************************************************************************
* Function Name  : OLED_DisplayOff
* Description    : رOLEDʾ
* Input          : None			
* Return         : None
*******************************************************************************/
void OLED_DisplayOff(void)
{
	OLED_WriteCmd(0x8D);
	OLED_WriteCmd(0x10);
	OLED_WriteCmd(0xAE);
}


/*******************************************************************************
* Function Name  : OLED_DrawPoint
* Description    : OLED(x,y)괦ĵ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 s: 1/0 = /
* Return         : None
*******************************************************************************/
/* OLED */
void OLED_DrawPoint(UINT8 x, UINT8 y, UINT8 s)
{
	UINT8 pos, page, temp=0;
	
	if(x>127 || y>63)
		return;
	
	page = 7-y/8;
	pos = y%8;
	temp = 1<<(7-pos);
	if(s)
		OLED_GRAM[x][page] |= temp;
	else
		OLED_GRAM[x][page] &= ~temp;
}



/*******************************************************************************
* Function Name  : OLED_Fill
* Description    : OLED(x1,y1)(x2,y2)Խߵĳϵĵ
* Input          : x1: 1ʾ
									 y1: 1ʾϵ
									 x2: 2ʾ
									 y2: 2ʾϵ
									 s: 1/0 = /
* Return         : None
*******************************************************************************/
void OLED_Fill(UINT8 x1, UINT8 x2, UINT8 y1, UINT8 y2, UINT8 s)
{
	UINT8 x, y;
	for(x=x1; x<x2; ++x)
	{
		for(y=y1; y<y2; ++y)
		{
			OLED_DrawPoint(x, y, s);
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_Fill
* Description    : OLED(x,y)ʾַ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 ch: ʾַ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowChar(UINT8 x, UINT8 y, UINT8 ch, UINT8 size)
{
	UINT8 temp, t, p;
	UINT8 y0 = y;
	UINT8 chsize = (size/8+((size%8)?1:0))*(size/2);
	ch = ch-' ';
	for(t=0; t<chsize; ++t)
	{
		if(size == 12)
			temp = asc2_1206[ch][t];
		else if(size == 16)
			temp = asc2_1608[ch][t];
		else if(size == 24)
			temp = asc2_2412[ch][t];
		else
			return;
		
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == size)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : MyPow
* Description    : ָ
* Input          : m: 
									 n: ָ
* Return         : result: 
*******************************************************************************/
UINT32 MyPow(UINT8 m, UINT8 n)
{
	UINT32 result = 1;
	while(n--)
		result *= m;
	return result;
}


/*******************************************************************************
* Function Name  : OLED_ShowNum
* Description    : OLED(x,y)ʾ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 num: ʾ
									 len: ֳ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowNum(UINT8 x, UINT8 y, UINT32 num, UINT8 len, UINT8 size)
{
	UINT8 t, temp;
	UINT8 enshow = 0;
	
	for(t=0; t<len; ++t)
	{
		temp = (num/MyPow(10,len-t-1))%10;
		if(enshow==0 && t<(len-1))
		{
			if(temp == 0)
			{
				OLED_ShowChar(x+(size/2)*t, y, ' ', size);
				continue;
			}
			else
				enshow = 1;
		}
		OLED_ShowChar(x+(size/2)*t, y, temp+'0', size);
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowString
* Description    : OLED(x,y)ʾַ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 *p: ַ׵ַ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowString(UINT8 x, UINT8 y, UINT8 *p, UINT8 size)
{
	while ((*p>=' ') && (*p<='~'))
	{
		if(x>(128-(size/2)))
		{
			x = 0;
			y += size;
		}
		if(y>(64-size))
		{
			y = x = 0;
			OLED_Clear();
		}
		
		OLED_ShowChar(x, y, *p, size);
		x += size/2;
		++p;
	}
}


/*******************************************************************************
* Function Name  : OLED_Init
* Description    : OLEDʼ
* Input          : None
* Return         : None
*******************************************************************************/
void OLED_Init(void)
{
	DelayMs(100);
	OLED_WriteCmd(0xAE);//--turn off oled panel
	OLED_WriteCmd(0xD5);//--set display clock divide ratio/oscillator frequency(0xd5);//--set display clock divide ratio/oscillator frequency
	OLED_WriteCmd(0x80);//--set divide ratio, Set Clock as 100 Frames/Sec	
	OLED_WriteCmd(0xA8);//--set multiplex ratio(1 to 64)
	OLED_WriteCmd(0x3F);//--1/64 duty
	OLED_WriteCmd(0xD3);//-set display offset	Shift Mapping RAM Counter (0x00~0x3F)
	OLED_WriteCmd(0x00);//-not offset
	
	OLED_WriteCmd(0x40);//Set VCOM Deselect Level

	OLED_WriteCmd(0x8D);//--set Charge Pump enable/disable
	OLED_WriteCmd(0x14);//--set(0x10) disable	
	OLED_WriteCmd(0x20);//-Set Page Addressing Mode (0x00/0x01/0x02)
	OLED_WriteCmd(0x02);//	
	OLED_WriteCmd(0xA1);//--Set SEG/Column Mapping     0xa0ҷ 0xa1
	OLED_WriteCmd(0xC0);//Set COM/Row Scan Direction   0xc0· 0xc8
	OLED_WriteCmd(0xDA);//--set com pins hardware configuration
	OLED_WriteCmd(0x12);
	
	OLED_WriteCmd(0x81);//--set contrast control register
	OLED_WriteCmd(0XEF); // Set SEG Output Current Brightness
	OLED_WriteCmd(0xD9);//--set pre-charge period
	OLED_WriteCmd(0xF1);//Set Pre-Charge as 15 Clocks & Discharge as 1 Clock
	OLED_WriteCmd(0xDB);//--set vcomh	
	OLED_WriteCmd(0x30);//--set vcomh	
	
	OLED_WriteCmd(0xA4);// Disable Entire Display On (0xa4/0xa5)
	OLED_WriteCmd(0xA6);// Disable Inverse Display On (0xa6/a7) 
	OLED_WriteCmd(0xAF);//--turn on oled panel

	OLED_Clear(); //ʼ
}


/*******************************************************************************
* Function Name  : OLED_ShowResPic
* Description    : OLED(x,y)ʾͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowResPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 64;
	for(t=0; t<charsize; ++t)
	{
		temp = RES_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 16)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowOhmUnit
* Description    : OLED(x,y)ʾ''ʶ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowOhmUnit(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 32;
	for(t=0; t<charsize; ++t)
	{
		temp = OHM_UNIT[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 16)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowCapPic
* Description    : OLED(x,y)ʾͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowCapPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 64;
	for(t=0; t<charsize; ++t)
	{
		temp = CAP_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 16)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowIndPic
* Description    : OLED(x,y)ʾͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowIndPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 64;
	for(t=0; t<charsize; ++t)
	{
		temp = IND_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 16)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowDioPic
* Description    : OLED(x,y)ʾͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowDioPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 64;
	for(t=0; t<charsize; ++t)
	{
		temp = DIO_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 16)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowPBJTPic
* Description    : OLED(x,y)ʾPBJTͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowPBJTPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = PBJT_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowNBJTPic
* Description    : OLED(x,y)ʾNBJTPͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowNBJTPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = NBJT_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowPJEFPic
* Description    : OLED(x,y)ʾPJEFͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowPJEFPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = PJFET_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowNJEFPic
* Description    : OLED(x,y)ʾNJEFͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowNJEFPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = NJFET_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowPEMOSPic
* Description    : OLED(x,y)ʾPEMOSͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowPEMOSPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = PEMOS_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowNEMOSPic
* Description    : OLED(x,y)ʾNEMOSͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowNEMOSPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = NEMOS_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowPDMOSPic
* Description    : OLED(x,y)ʾPDMOSͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowPDMOSPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = PDMOS_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowNDMOSPic
* Description    : OLED(x,y)ʾNDMOSͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowNDMOSPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = NDMOS_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowPEMOSDIOPic
* Description    : OLED(x,y)ʾPEMOSDIOͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowPEMOSDIOPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = PEMOSDIO_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}


/*******************************************************************************
* Function Name  : OLED_ShowPEMOSDIOPic
* Description    : OLED(x,y)ʾNEMOSDIOͼƬ
* Input          : x: ꣬ʾ
									 y: ꣬ʾϵ
									 size: ַĸ߶
* Return         : None
*******************************************************************************/
void OLED_ShowNEMOSDIOPic(UINT8 x, UINT8 y)
{
	UINT8 temp, t, p; 
	UINT8 y0 = y;
	UINT8 charsize = 72;
	for(t=0; t<charsize; ++t)
	{
		temp = NEMOSDIO_PIC[t];
		for(p=0; p<8; ++p)
		{
			if(temp&0x80)
				OLED_DrawPoint(x, y, 1);
			else
				OLED_DrawPoint(x, y, 0);
			
			temp <<= 1;
			++y;
			if((y-y0) == 32)
			{
				y = y0;
				++x;
				break;
			}
		}
	}
}
